/*
 ## Cypress USB 3.0 Platform source file (cyfxuartlpdmamode.c)
 ## ===========================
 ##
 ##  Copyright Cypress Semiconductor Corporation, 2010-2018,
 ##  All Rights Reserved
 ##  UNPUBLISHED, LICENSED SOFTWARE.
 ##
 ##  CONFIDENTIAL AND PROPRIETARY INFORMATION
 ##  WHICH IS THE PROPERTY OF CYPRESS.
 ##
 ##  Use of this file is governed
 ##  by the license agreement included in the file
 ##
 ##     <install>/license/license.txt
 ##
 ##  where <install> is the Cypress software
 ##  installation root directory path.
 ##
 ## ===========================
*/

/* This file implements a simple UART (DMA mode) loopback application example */

/*
   This example illustrates a loopback mechanism between the UART RX and UART TX
   using DMA MANUAL channel. The loopback is achieved with the help of a DMA MANUAL
   channel. DMA MANUAL channel is created between the UART producer socket and the
   UART consumer socket.

   Upon every reception of data in the DMA buffer from the host, the CPU is signalled
   using DMA callbacks. The CPU then commits the DMA buffer received so that the data
   is transferred to the consumer socket.

   Note: Since the UART RX will commit the data to UART TX only after the DMA buffer
   is filled, the remote UART receiver will only get the loopback data after receiving
   CY_FX_UART_DMA_BUF_SIZE bytes.
 */

#include <cyu3system.h>
#include <cyu3os.h>
#include <cyu3error.h>
#include <cyu3uart.h>
#include "cyu3gpio.h"
#include <cyu3utils.h>
#include "cyu3usb.h"
#include "uart_regs.h"
#define REG32(address)		(*((volatile UINT*)(address)))
CyBool_t UART_ready      = CyFalse;
CyU3PDmaBuffer_t inBuf_p, outBuf_p;
CyU3PDmaBuffer_t buf_p,obuf_p;

#define CY_FX_UARTLP_THREAD_STACK       (0x0400)  /* UART application thread stack size */
#define CY_FX_UARTLP_THREAD_PRIORITY    (8)       /* UART application thread priority */
#define CY_FX_UART_DMA_TX_SIZE	(0)               /* DMA transfer size */
#define CY_FX_UART_DMA_BUF_SIZE (16)              /* Buffer size */

#define not_override

CyU3PThread UartLpAppThread;	                  /* UART Example application thread structure */
//CyU3PDmaChannel glUartLpChHandle;                 /* DMA Channel handle */
CyU3PDmaChannel glUartRXChHandle;
CyU3PDmaChannel glUartTXChHandle;

uint16_t prod, cons =0;

/* Application error handler */
void
CyFxAppErrorHandler (
        CyU3PReturnStatus_t apiRetStatus    /* API return status */
        )
{
    /* Application failed with the error code apiRetStatus */

    /* Add custom debug or recovery actions here */

    /* Loop indefinitely */
    for (;;)
    {
        /* Thread sleep : 100 ms */
        CyU3PThreadSleep (100);
    }
}
#ifdef not_override
void
CyFxUartLpDmaCallback (
        CyU3PDmaChannel   *chHandle, /* Handle to the DMA channel. */
        CyU3PDmaCbType_t  type,      /* Callback type.             */
        CyU3PDmaCBInput_t *input)    /* Callback status.           */
{
    CyU3PReturnStatus_t status;


    if (type == CY_U3P_DMA_CB_PROD_EVENT)
    {
    	prod++;
        /* This is a produce event notification to the CPU. This notification is
         * received upon reception of every buffer. The buffer will not be sent
         * out unless it is explicitly committed. The call shall fail if there
         * is any application error. */
    if(UART_ready == CyFalse){
    	status = CyU3PDmaChannelGetBuffer (&glUartRXChHandle, &inBuf_p, CYU3P_WAIT_FOREVER);
    	            if (status != CY_U3P_SUCCESS)
    	            {

    	                    CyFxAppErrorHandler(status);
    	                }
    	            else{
    	            	        /* Set UART Tx and Rx transfer Size to infinite */
    	            	       status = CyU3PUartTxSetBlockXfer(16);//0xFFFFFFFF
    	            	        if (status != CY_U3P_SUCCESS)
    	            	        {
    	            	            /* Error handling */
    	            	            CyFxAppErrorHandler(status);
    	            	        }
    	            	        /* Set DMA Channel transfer size */
    	            	       status = CyU3PDmaChannelSetXfer (&glUartTXChHandle, 16);
    	            	        if (status != CY_U3P_SUCCESS)
    	            	        {
    	            	            /* Error handling */
    	            	            CyFxAppErrorHandler(status);
    	            	        }
    	            	        if(status == CY_U3P_SUCCESS )
    	            	             UART_ready = CyTrue;


    	            }

    	}

//        status = CyU3PDmaChannelCommitBuffer (chHandle, input->buffer_p.count, 0);
//        if (status != CY_U3P_SUCCESS)
//        {
//            CyFxAppErrorHandler (status);
//        }
    }

    if (type == CY_U3P_DMA_CB_CONS_EVENT)
    {
    	cons++;
        /* This is a produce event notification to the CPU. This notification is
         * received upon reception of every buffer. The buffer will not be sent
         * out unless it is explicitly committed. The call shall fail if there
         * is any application error. */
    	 /* Set UART Tx and Rx transfer Size to infinite */
    	    	            	       status = CyU3PUartRxSetBlockXfer(16);//0xFFFFFFFF
    	    	            	        if (status != CY_U3P_SUCCESS)
    	    	            	        {
    	    	            	            /* Error handling */
    	    	            	            CyFxAppErrorHandler(status);
    	    	            	        }
    	    	            	        /* Set DMA Channel transfer size */
    	    	            	       status = CyU3PDmaChannelSetXfer (&glUartRXChHandle, 16);
    	    	            	        if (status != CY_U3P_SUCCESS)
    	    	            	        {
    	    	            	            /* Error handling */
    	    	            	            CyFxAppErrorHandler(status);
    	    	            	        }
//        status = CyU3PDmaChannelCommitBuffer (chHandle, input->buffer_p.count, 0);
//        if (status != CY_U3P_SUCCESS)
//        {
//            CyFxAppErrorHandler (status);
//        }
    }


}
#endif

#ifndef not_override
void SendDebugMessage(CyU3PDmaBuffer_t* buffer, uint8_t len)
{
	//CyU3PDmaBuffer_t buf_p;
	CyU3PReturnStatus_t status;
//	buf_p.buffer = buffer;
//	buf_p.status = 0;
//	buf_p.size   = 32;
//	buf_p.count  = len;

	CyU3PDmaChannelReset(&glUartTXChHandle);
	CyU3PGpioSetValue (38, CyTrue);
	status = CyU3PDmaChannelSetupSendBuffer(&glUartTXChHandle, &buffer);
	if(status != CY_U3P_SUCCESS)
		CyFxAppErrorHandler(status);

	CyU3PBusyWait(86.8*len);
	//CyU3PDmaChannelReset(&glChHandleUsbtoUart);
	//status = CyU3PDmaChannelSetXfer (&glUartTXChHandle,0);
	//if(status != CY_U3P_SUCCESS)
		//CyFxAppErrorHandler(status);

	CyU3PGpioSetValue (38, CyFalse);
	CyU3PUartTxSetBlockXfer(16);
}



//cy_tech
uint8_t count = 0;
uint8_t len = 16;
#endif

#ifndef not_override
void UartIsr (
        CyU3PUartEvt_t evt,             /**< Type of event that occured. */
        CyU3PUartError_t error          /**< Error/status code associated with CY_U3P_UART_EVENT_ERROR events. */
        )
{
	CyU3PReturnStatus_t status;
	uint8_t glRXDataBuffer[16];
//	CyU3PDmaBuffer_t buf_p;
//	buf_p.buffer = glRXDataBuffer;
//    buf_p.status = 0;
//    buf_p.size   = 16;
//    buf_p.count  = 0;//len

	if (evt == CY_U3P_UART_EVENT_RX_DONE)
	{
		UART_ready = CyTrue;


	/*	//************************** Fixed Buffer ***************************
	    uint8_t buf[len];
		int i;
		for( i = 0; i < len; i++)
			buf[i] = i+65;
		SendDebugMessage(buf, len);
    */

   /* //******************  DMA Buffer / CyU3PDmaChannelSetupRecvBuffer   **************************
   	   status = CyU3PDmaChannelSetupRecvBuffer(&glUartRXChHandle, &buf_p);
   	   status = CyU3PDmaChannelWaitForCompletion( &glUartRXChHandle,CYU3P_WAIT_FOREVER);// , 0x7ffff CYU3P_WAIT_FOREVER   CYU3P_NO_WAIT
   */

	  //****************** DMA Buffer / CyU3PDmaChannelGetBuffer  ***************************
//	status = CyU3PDmaChannelSetWrapUp(&glUartRXChHandle);
//	status = CyU3PDmaChannelGetBuffer(&glUartRXChHandle, &buf_p, CYU3P_NO_WAIT);//  CYU3P_WAIT_FOREVER
//	SendDebugMessage(buf_p.buffer, len);
//	CyU3PDmaChannelDiscardBuffer (&glUartRXChHandle);

	//CyU3PUartRxSetBlockXfer(16);
	 }
}
#endif



/* This function initializes the UART module */
void
CyFxUartLpApplnInit (void)
{
    CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;
 	CyU3PGpioSimpleConfig_t  gpioConfig;
	CyU3PUartConfig_t        uartConfig;
    CyU3PDmaChannelConfig_t  dmaConfig;

	//*********************************************************
 	CyU3PGpioClock_t gpioClock;
	//   CyU3PGpioComplexConfig_t ppsGpioConfigComplex;

	     	 /* GPIO */
	     		    /* Init the GPIO module. The GPIO block will be running
	     		     * with a fast clock at SYS_CLK / 2 and slow clock is not
	     		     * used. For the DVK, the SYS_CLK is running at 403 MHz.*/
	     	gpioClock.fastClkDiv = 2;
	     	gpioClock.slowClkDiv = 0;
	     	gpioClock.simpleDiv  = CY_U3P_GPIO_SIMPLE_DIV_BY_2;
	     	gpioClock.clkSrc     = CY_U3P_SYS_CLK;
	     	gpioClock.halfDiv    = 0;

	     	/* Initialize Gpio interface */
	     	CyU3PGpioInit (&gpioClock, NULL);

	         /* Configure GPIO38(DQ21_pin28/J7) as an output */  // UART Selection
	         gpioConfig.outValue    = CyFalse;
	         gpioConfig.driveLowEn  = CyTrue;
	         gpioConfig.driveHighEn = CyTrue;
	         gpioConfig.inputEn     = CyFalse;
	         gpioConfig.intrMode    = CY_U3P_GPIO_NO_INTR;

	         apiRetStatus		   = CyU3PGpioSetSimpleConfig(38, &gpioConfig);
	         if (apiRetStatus != CY_U3P_SUCCESS)
	         {
	               /* Error handling */
	               CyU3PDebugPrint (4, "CyU3PGpioSetSimpleConfig failed, error code = %d\n",
	                       apiRetStatus);
	               CyFxAppErrorHandler(apiRetStatus);
	         }
	 //**********************************************************************************

    /* Initialize the UART module */
    apiRetStatus = CyU3PUartInit ();
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        /* Error handling */
        CyFxAppErrorHandler(apiRetStatus);
    }

    /* Configure the UART
       Baudrate = 115200, One stop bit, No parity, Hardware flow control enabled.
     */
    CyU3PMemSet ((uint8_t *)&uartConfig, 0, sizeof(uartConfig));
    uartConfig.baudRate = CY_U3P_UART_BAUDRATE_115200;
    uartConfig.stopBit  = CY_U3P_UART_ONE_STOP_BIT;
    uartConfig.parity   = CY_U3P_UART_NO_PARITY;
    uartConfig.flowCtrl = CyFalse;
    uartConfig.txEnable = CyTrue;
    uartConfig.rxEnable = CyTrue;
    uartConfig.isDma    = CyTrue; /* DMA mode */

#ifndef not_override
    /* Set the UART configuration */
    apiRetStatus = CyU3PUartSetConfig (&uartConfig, UartIsr);//NULL
    if (apiRetStatus != CY_U3P_SUCCESS )
    {
        /* Error handling */
        CyFxAppErrorHandler(apiRetStatus);
    }
#else
    apiRetStatus = CyU3PUartSetConfig (&uartConfig, NULL);//NULL
    if (apiRetStatus != CY_U3P_SUCCESS )
    {
        /* Error handling */
        CyFxAppErrorHandler(apiRetStatus);
    }
#endif
#ifndef not_override
	//CyU3PRegisterUartCallBack((CyU3PUartIntrCb_t)UartIsr);
	UART->lpp_uart_config &= ~(CY_U3P_LPP_UART_ENABLE);
	//		REGAB(0xE000080C) = 0x1;//UART INTR MASK
	//UART -> lpp_uart_intr_mask = 0x1;
	//cy_tech

	REG32(0xE000080C) = 0x1;
	UART->lpp_uart_config |= CY_U3P_LPP_UART_ENABLE;
#endif

    //**********************************************************************************

#ifndef not_override

    /* Create a DMA Manual channel between UART producer socket
       and UART consumer socket */
        CyU3PMemSet ((uint8_t *)&dmaConfig, 0, sizeof(dmaConfig));
       	dmaConfig.size          = CY_FX_UART_DMA_BUF_SIZE;
    /* No buffers need to be allocated as this will be used
         * only in override mode. */
        dmaConfig.count    		 = 0;                           //this means that dma channel is to be used n override mode
        dmaConfig.dmaMode        = CY_U3P_DMA_MODE_BYTE;
        dmaConfig.notification   = 0;//CY_U3P_DMA_CB_PROD_EVENT;//
        dmaConfig.cb             = NULL;//CyFxUartLpDmaCallback;//
        dmaConfig.prodHeader     = 0;
        dmaConfig.prodFooter     = 0;
        dmaConfig.consHeader     = 0;
        dmaConfig.prodAvailCount = 0;


        /* Create a channel to write to the uart. */
        dmaConfig.prodSckId = CY_U3P_CPU_SOCKET_PROD;
        dmaConfig.consSckId = CY_U3P_LPP_SOCKET_UART_CONS;
        apiRetStatus = CyU3PDmaChannelCreate (&glUartTXChHandle, CY_U3P_DMA_TYPE_MANUAL_OUT, &dmaConfig);
        if (apiRetStatus != CY_U3P_SUCCESS)
        {

        }

        //dmaConfig.notification   = CY_U3P_DMA_CB_PROD_EVENT;//
        //dmaConfig.cb             = CyFxUartLpDmaCallback;//
        /* Create a channel to read from the uart. */
        dmaConfig.prodSckId = CY_U3P_LPP_SOCKET_UART_PROD;
        dmaConfig.consSckId = CY_U3P_CPU_SOCKET_CONS;
        apiRetStatus = CyU3PDmaChannelCreate (&glUartRXChHandle,  CY_U3P_DMA_TYPE_MANUAL_IN, &dmaConfig);



    /* Set UART Tx and Rx transfer Size to infinite */
    apiRetStatus = CyU3PUartTxSetBlockXfer(16);//0xFFFFFFFF
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        /* Error handling */
        CyFxAppErrorHandler(apiRetStatus);
    }

    //This means after one byte is received, the ISR is called.
    apiRetStatus = CyU3PUartRxSetBlockXfer(16);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        /* Error handling */
        CyFxAppErrorHandler(apiRetStatus);
    }

//    /* Set DMA Channel transfer size */
//    apiRetStatus = CyU3PDmaChannelSetXfer (&glUartTXChHandle, 0);           //not needed for override mode
//    if (apiRetStatus != CY_U3P_SUCCESS)
//    {
//        /* Error handling */
//        CyFxAppErrorHandler(apiRetStatus);
//    }
//
    /* Set DMA Channel transfer size */
//    apiRetStatus = CyU3PDmaChannelSetXfer (&glUartRXChHandle, 0);     //not needed for override mode
//    if (apiRetStatus != CY_U3P_SUCCESS)
//    {
//        /* Error handling */
//        CyFxAppErrorHandler(apiRetStatus);
//    }
#else
    /* Create a DMA Manual channel between UART producer socket
           and UART consumer socket */
            CyU3PMemSet ((uint8_t *)&dmaConfig, 0, sizeof(dmaConfig));
           	dmaConfig.size          = CY_FX_UART_DMA_BUF_SIZE;
        /* No buffers need to be allocated as this will be used
             * only in override mode. */
            dmaConfig.count    		 = 4;
            dmaConfig.dmaMode        = CY_U3P_DMA_MODE_BYTE;
            dmaConfig.notification   = CY_U3P_DMA_CB_PROD_EVENT | CY_U3P_DMA_CB_CONS_EVENT |CY_U3P_DMA_CB_XFER_CPLT;
            dmaConfig.cb             = CyFxUartLpDmaCallback;//
            dmaConfig.prodHeader     = 0;
            dmaConfig.prodFooter     = 0;
            dmaConfig.consHeader     = 0;
            dmaConfig.prodAvailCount = 0;


            /* Create a channel to write to the uart. */
            dmaConfig.prodSckId = CY_U3P_CPU_SOCKET_PROD;
            dmaConfig.consSckId = CY_U3P_LPP_SOCKET_UART_CONS;
            apiRetStatus = CyU3PDmaChannelCreate (&glUartTXChHandle, CY_U3P_DMA_TYPE_MANUAL_OUT, &dmaConfig);
            if (apiRetStatus != CY_U3P_SUCCESS)
            {

            }

            //dmaConfig.notification   = CY_U3P_DMA_CB_PROD_EVENT;//
            //dmaConfig.cb             = CyFxUartLpDmaCallback;//
            /* Create a channel to read from the uart. */
            dmaConfig.prodSckId = CY_U3P_LPP_SOCKET_UART_PROD;
            dmaConfig.consSckId = CY_U3P_CPU_SOCKET_CONS;
            apiRetStatus = CyU3PDmaChannelCreate (&glUartRXChHandle,  CY_U3P_DMA_TYPE_MANUAL_IN, &dmaConfig);



//        /* Set UART Tx and Rx transfer Size to infinite */
//        apiRetStatus = CyU3PUartTxSetBlockXfer(16);//0xFFFFFFFF
//        if (apiRetStatus != CY_U3P_SUCCESS)
//        {
//            /* Error handling */
//            CyFxAppErrorHandler(apiRetStatus);
//        }

        //This means after one byte is received, the ISR is called.
        apiRetStatus = CyU3PUartRxSetBlockXfer(16);
        if (apiRetStatus != CY_U3P_SUCCESS)
        {
            /* Error handling */
            CyFxAppErrorHandler(apiRetStatus);
        }

//        /* Set DMA Channel transfer size */
//        apiRetStatus = CyU3PDmaChannelSetXfer (&glUartTXChHandle, 16);
//        if (apiRetStatus != CY_U3P_SUCCESS)
//        {
//            /* Error handling */
//            CyFxAppErrorHandler(apiRetStatus);
//        }

        /* Set DMA Channel transfer size */
        apiRetStatus = CyU3PDmaChannelSetXfer (&glUartRXChHandle, 16);
        if (apiRetStatus != CY_U3P_SUCCESS)
        {
            /* Error handling */
            CyFxAppErrorHandler(apiRetStatus);
        }
#endif


//
//        /*Example Code*/
//        uint8_t buf[10];
//        int i;
//    	for( i = 0; i < 10; i++)
//    		buf[i] = i+65;
//    	SendDebugMessage(buf, 10);



}

/* Entry function for the UartLpAppThread */
void
UartLpAppThread_Entry (
        uint32_t input)
{
    /* Initialize the UART Example Application */
    CyFxUartLpApplnInit();

    //CyU3PUartSetTimeout (100, 5000);

    CyU3PReturnStatus_t status;


    for (;;)
    {
#ifdef not_override
    	// CyU3PThreadSleep (1000);
    	if(UART_ready)
    	{
    	      status = CyU3PDmaChannelGetBuffer (&glUartTXChHandle, &outBuf_p, CYU3P_WAIT_FOREVER);
    	            if (status != CY_U3P_SUCCESS)
    	            {
    	            	CyFxAppErrorHandler(status);
    	            }

    	            CyU3PMemCopy (outBuf_p.buffer, inBuf_p.buffer, inBuf_p.count);   //copying data to another buffer


    		        status = CyU3PDmaChannelDiscardBuffer (&glUartRXChHandle);
    		              if (status != CY_U3P_SUCCESS)
    		              {
    		            	  CyFxAppErrorHandler(status);
    		              }

    		        status = CyU3PDmaChannelCommitBuffer (&glUartTXChHandle, inBuf_p.count, 0);
    		                         if (status != CY_U3P_SUCCESS)
    		                         {
    		                        	 CyFxAppErrorHandler(status);
    		                         }



    		                               //This means after one byte is received, the ISR is called.
    		                               status = CyU3PUartRxSetBlockXfer(16);
    		                               if (status != CY_U3P_SUCCESS)
    		                               {
    		                                   /* Error handling */
    		                                   CyFxAppErrorHandler(status);
    		                               }



    		                               /* Set DMA Channel transfer size */
    		                               status = CyU3PDmaChannelSetXfer (&glUartRXChHandle, 16);
    		                               if (status != CY_U3P_SUCCESS)
    		                               {
    		                                   /* Error handling */
    		                                   CyFxAppErrorHandler(status);
    		                               }
    		                         if( status == CY_U3P_SUCCESS)

    		                                UART_ready = CyFalse;
    	}
#else
    	if(UART_ready)
    	{
    			buf_p.buffer = buffer;
    			buf_p.status = 0;
    			buf_p.size   = 16;
    			buf_p.count  = 16;
    		status = CyU3PDmaChannelSetupRecvBuffer(&glUartRXChHandle, &buf_p);
    	      if (status != CY_U3P_SUCCESS)
    	    		                               {
    	    		                                   /* Error handling */
    	    		                                   CyFxAppErrorHandler(status);
    	    		                               }
    	      status = CyU3PDmaChannelWaitForCompletion( &glUartRXChHandle,CYU3P_WAIT_FOREVER);
    	      if (status != CY_U3P_SUCCESS)
    	      	    		                               {
    	      	    		                                   /* Error handling */
    	      	    		                                   CyFxAppErrorHandler(status);
    	      	    		                               }

    	      CyU3PMemCopy (obuf_p.buffer, buf_p.buffer, buf_p.count);  //copying data to another buffer

    	      status = CyU3PDmaChannelDiscardBuffer (&glUartRXChHandle);
    	        		              if (status != CY_U3P_SUCCESS)
    	        		              {
    	        		            	  CyFxAppErrorHandler(status);
    	        		              }



    	      SendDebugMessage(obuf_p, obuf_p.count);


    	      apiRetStatus = CyU3PUartRxSetBlockXfer(16);
    	        if (apiRetStatus != CY_U3P_SUCCESS)
    	        {
    	            /* Error handling */
    	            CyFxAppErrorHandler(apiRetStatus);
    	        }

    	        UART_ready = CyFalse;


    	}
#endif

    	/*
    	 if(UART_ready)
    	 {
//    		uint8_t buf[len];
//    		int i;
//    		for( i = 0; i < len; i++)
//    			buf[i] = i+65;
//    		 SendDebugMessage(buf, len);

    		 status = CyU3PDmaChannelSetupRecvBuffer(&glUartRXChHandle, &buf_p);
    		 status = CyU3PDmaChannelWaitForCompletion( &glUartRXChHandle,CYU3P_WAIT_FOREVER);// , 0x7ffff CYU3P_WAIT_FOREVER   CYU3P_NO_WAIT
    		//CyU3PDmaChannelGetBuffer (&glUartRXChHandle, &buf_p);
    		SendDebugMessage(buf_p.buffer, len);
    		CyU3PDmaChannelDiscardBuffer (&glUartRXChHandle);


    		UART_ready = CyFalse;
    	 }
*/

        /* No operation in the thread */
        CyU3PThreadSleep (100);
    }
}

/* Application define function which creates the threads. */
void
CyFxApplicationDefine (
        void)
{
    void *ptr = NULL;
    uint32_t retThrdCreate = CY_U3P_SUCCESS;

    /* Allocate the memory for the threads */
    ptr = CyU3PMemAlloc (CY_FX_UARTLP_THREAD_STACK);

    /* Create the thread for the application */
    retThrdCreate = CyU3PThreadCreate (&UartLpAppThread,           /* UART Example App Thread structure */
                          "21:UART_loopback_DMA_mode",             /* Thread ID and Thread name */
                          UartLpAppThread_Entry,                   /* UART Example App Thread Entry function */
                          0,                                       /* No input parameter to thread */
                          ptr,                                     /* Pointer to the allocated thread stack */
                          CY_FX_UARTLP_THREAD_STACK,               /* UART Example App Thread stack size */
                          CY_FX_UARTLP_THREAD_PRIORITY,            /* UART Example App Thread priority */
                          CY_FX_UARTLP_THREAD_PRIORITY,            /* UART Example App Thread priority */
                          CYU3P_NO_TIME_SLICE,                     /* No time slice for the application thread */
                          CYU3P_AUTO_START                         /* Start the Thread immediately */
                          );

    /* Check the return code */
    if (retThrdCreate != 0)
    {
        /* Thread Creation failed with the error code retThrdCreate */

        /* Add custom recovery or debug actions here */

        /* Application cannot continue */
        /* Loop indefinitely */
        while(1);
    }
}

/*
 * Main function
 */
int
main (void)
{
    CyU3PIoMatrixConfig_t io_cfg;
    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;

    /* Initialize the device */
    status = CyU3PDeviceInit (0);
    if (status != CY_U3P_SUCCESS)
    {
        goto handle_fatal_error;
    }

    /* Initialize the caches. Enable both Instruction and Data Caches. */
    status = CyU3PDeviceCacheControl (CyTrue, CyFalse, CyFalse);   //16 byte aligned dma buffer
    if (status != CY_U3P_SUCCESS)
    {
        goto handle_fatal_error;
    }

    /* Configure the IO matrix for the device. On the FX3 DVK board, the COM port 
     * is connected to the IO(53:56). This means that either DQ32 mode should be
     * selected or lppMode should be set to UART_ONLY. Here we are choosing
     * UART_ONLY configuration. */
    CyU3PMemSet ((uint8_t *)&io_cfg, 0, sizeof(io_cfg));
    io_cfg.isDQ32Bit = CyTrue;
    io_cfg.s0Mode = CY_U3P_SPORT_INACTIVE;
    io_cfg.s1Mode = CY_U3P_SPORT_INACTIVE;
    io_cfg.useUart   = CyTrue;
    io_cfg.useI2C    = CyFalse;
    io_cfg.useI2S    = CyFalse;
    io_cfg.useSpi    = CyFalse;
    io_cfg.lppMode   = CY_U3P_IO_MATRIX_LPP_DEFAULT;
    /* No GPIOs are enabled. */
    io_cfg.gpioSimpleEn[0]  = 0;
    io_cfg.gpioSimpleEn[1]  = 0x00000040;
    io_cfg.gpioComplexEn[0] = 0;
    io_cfg.gpioComplexEn[1] = 0;
    status = CyU3PDeviceConfigureIOMatrix (&io_cfg);
    if (status != CY_U3P_SUCCESS)
    {
        goto handle_fatal_error;
    }

    /* This is a non returnable call for initializing the RTOS kernel */
    CyU3PKernelEntry ();

    /* Dummy return to make the compiler happy */
    return 0;

handle_fatal_error:
    /* Cannot recover from this error. */
    while (1);

}

/* [ ] */




